/* Here we build the bdd, corresponding to the statement that in the arbitrary
   graph of the order N there is a fully connected sub-graph of the order m
   or a fully disconnected subgraph of the order n.
   Then we iteratively search for counter-examples to that statement,
   considering graphs of increasing size N (starting from N=max(m,n)). Once
   we reach the smallest N such that there are no counter-examples -- woila !
   That N is the elusive Ramsey number R(m,n).
*/

// (c) 2012 Konstantin L. Metlov <metlov@fti.dn.ua>
//
// You can redistribute and/or modify this file under the terms of the GNU
// General Public License as published by the Free Software Foundation, eith
// version 3 of the License, or (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

#include <cstdlib>
#include <cmath>
#include <cassert>
#include <iostream>
#include <iomanip>
#include <bdd.h>

using namespace std;

void first_comb(char* c, int n, int k);
bool next_comb(char* c, int n, int k);
void comb_to_edges(char* c, int n, int m, int* e);
bdd there_is(bool connected, int N, int m);

int hiusednodeswatermark=0;
int lousednodeswatermark=0;
void my_gbchandler(int pre, bddGbcStat *s);

void solPrint(char* varset, int size) {
for (int i=0; i<size; i++) cerr<<(varset[i]<0?'?':(varset[i]>0?'+':'-'));
cerr<<endl;
};

int main(int argc, char *argv[]) {

   const int m=3;
   const int n=4;

   // look for Ramsey number
   cerr << "Looking for Ramsey number R("<<m<<","<<n<<")="<<endl;
   int N=max(m,n);
   double non_satisfying;
   do {
      bdd_init(8000000,1000000);  // about 270m
      //bdd_init(70000000,8000000);   // about 2253m (around max for 32 bit arch)
      bdd_setmaxincrease(0);
      bdd_gbc_hook(my_gbchandler); // monitor garbage collection

      const int Ne=N*(N-1)/2;
      cerr << "Considering complete "<< N << "-graph, "<<
        "which has "<<Ne<<" edges."<<endl;
      bdd_setvarnum(Ne);
      bdd ramsey = (there_is(true,N,m) | there_is(false,N,n));
      non_satisfying=bdd_satcount(!ramsey);
      cout << "There are " << non_satisfying << " not satisfying graphs"<< endl;
      if (non_satisfying>0) {
         cerr << "The number " << N << " is NOT the Ramsey number R("<<m<<","<<n<<"). ";
         // cerr << endl << "Here are the counter-examples:" << endl;
         // bdd_allsat(ramsey,solPrint);
         cerr << "Continuing search."<< endl;
      } else {
         cerr << "Eureka ! R("<<m<<","<<n<<")=" << N << endl;
      };
      N++;
      bdd_done();
   } while (non_satisfying>0);
};

/*
   Initializes combinations generator by lexicographically smallest combination
   that is [0,1,...,k-1]. The arrat c must have at least k elements.
*/
void first_comb(char* c, int n, int k) {
   assert(k<=n);
   assert(k!=0);
   for (int i=0; i<k; i++) c[i]=i;
};

/*
   Advance to the next combination in lexicographic order.
   Algorithm from D.E. Knuth, "The art of computer programming",
   pre-fascicle 3a, section 7.2.1.3
*/
bool next_comb(char * c, int n, int k) {
   // easy case
   if (c[k-1]<n-1) {
      c[k-1]++;
      return true;
   };
   // find element to increase
   int i=k-1;
   while ((i>=0)&&(c[i]==n-k+i)) i--;
   if (i<0) return false; // reached the last combination
   c[i]++;
   for(i++;i<k;i++) c[i]=c[i-1]+1; // carry
   return true;
};

/*
   Edges in a particular subgraph, corresponding to a paricular combination
   of deleted vertices. The array e must have space for m*(m-1)/2 elements.
*/
void comb_to_edges(char* c, int n, int m, int* e) {
   bool cm[n];
   for(int i=0;i<n;i++) cm[i]=false;
   for(int i=0;i<m;i++) cm[(int)c[i]]=true;
   int ce=0;
   int cme=0;
   for (int i=0;i<n-1;i++)
      for (int j=n-1; j>=i+1;j--) {
         if (cm[i]&&cm[j]) e[cme++]=ce;
         ce++;
      };
   assert(cme==m*(m-1)/2);
};

bdd there_is(bool connected, int N, int m) {
   bdd there_is=bdd_false();
   char c[m];
   first_comb(c,N,m);
   do {
     const int nme=m*(m-1)/2;
     int e[nme];
     comb_to_edges(c, N, m, e);
     bdd is=connected?bdd_true():bdd_false();
     for (int i=0; i<nme; i++)  {
        if (connected)
           is &= bdd_ithvarpp(e[i]);
        else
           is |= bdd_ithvarpp(e[i]);
     }
     if (!connected) is=!is;
     there_is |= is;
   } while (next_comb(c,N,m));
   return there_is;
}

// garbage collection callback to collect statistics
void my_gbchandler(int pre, bddGbcStat *s) {
  int used=(s->nodes)-(s->freenodes);
  if (pre) {
    if (used>hiusednodeswatermark) hiusednodeswatermark=used;
  } else {
    if (used>lousednodeswatermark) lousednodeswatermark=used;
  }
  if (!pre) {
    int nodes=s->nodes;
    int softpct=int(double(hiusednodeswatermark)/nodes*100);
    int hardpct=int(double(lousednodeswatermark)/nodes*100);
    printf("GC #%d: %d nodes / %d free (usage: %d%% soft, %d%% hard)",
           s->num, s->nodes, s->freenodes, softpct, hardpct);
    printf(" [ %.1fs / %.1fs ]\n",
           (float)s->time/(float)(CLOCKS_PER_SEC),
           (float)s->sumtime/(float)CLOCKS_PER_SEC);
  }
}
